# KUPLOT 3.0 Universal plot program
# ================================================================
#
# This is the Makefile for KUPLOT on platforms using f77. 
# Uncomment the compiler options etc. for your platform from
# the list below. If you platform is not listed and you succeed
# installing the program, please send us a short notice.
#
# (c) Th. Proffen    (proffen@rsc.anu.edu.au)
#     R.B. Neder     (neder@yoda.kri.physik.uni-muenchen.de)
#

# Where to install the binary and help file and a backup location

BINDIR  = $(HOME)/bin
BACKDIR = $(HOME)/develop/discus-3.0/kuplot

# Your compiler names and install program

F77     = f77
CC      = cc
INSTALL =  install -c
#INSTALL = cp

# Here are the platform specific definitions
# If you want line editing add -DREADLINE on
# the CFLAGS line. If you have trouble with interactiv
# input, remove -DREADLINE.

##########################
# SUN Sparc Workstation
##########################
FFLAGS  = -Nl60 -C -O
CFLAGS  = -DREADLINE -I/usr/openwin/include -w
#
#use the line below line in SUNOS 4.x systems with X11R5
LDFLAGS = -lm -lV77 -lX11
#
#use the line below line in SUNOS 5.x systems with X11R5
#LDFLAGS = -lm -lV77 -lX11 -lsocket
#
SYSOBJ  = appl_sun.o sys_sun.o
SYSSRC  = appl_sun.f sys_sun.f

##########################
# SGI Power Challange
##########################
#FFLAGS  = -g
#CFLAGS  = -w -g
#LDFLAGS = -lX11
#SYSOBJ  = sys_hpu.o appl_hpu.o

##########################
# HP/UX Workstation
##########################
#FFLAGS  = -C +E1 -O +Obb1024 
#CFLAGS  = -DREADLINE -w -I/usr/include/X11R5
#LDFLAGS = /usr/lib/X11R5/libX11.sl
#SYSOBJ  = sys_hpu.o appl_hpu.o

##########################
# DEC Alpha
##########################
#FFLAGS  = -O
#CFLAGS  = -DREADLINE -w 
#LDFLAGS = -lX11
#SYSOBJ  = appl_sun.o sys_sun.o

##########################
# There should be no need to change anything below this line

OBJ = 	kuplot.o dkdo.o macro.o errlist.o random.o upd_par.o \
	sup.o kdo.o config.o err_appl.o learn.o exit.o help.o plot.o \
	para.o math.o load.o save.o clip.o fit.o graph.o ps.o x11.o \
	tek.o hpgl.o show.o color.o frame.o calc.o draw.o \
	cx11.o ctek.o csup.o readline.o $(SYSOBJ)

kuplot: $(OBJ)
	$(F77) $(FFLAGS) -o kuplot $(OBJ) $(LDFLAGS)

clean: 
	rm -f kuplot $(OBJ) 

install: kuplot kuplot.hlp
	$(INSTALL) kuplot $(BINDIR)
	$(INSTALL) kuplot.hlp $(BINDIR)

backup: 
	tar -cvf kuplot.tar *.f *.h c*.c sys_*.c *.inc *.hlp *ake* CHANGES.LOG
	gzip kuplot.tar
	mv kuplot.tar.gz $(BACKDIR)

# Here are the dependencies created with 'mkmf'
# Do not edit or delete
############################################################################
############################################################################
# DO NOT DELETE
###
appl_f2c.o: envir.inc
appl_hpu.o: envir.inc
appl_sun.o: envir.inc
calc.o: kuplot.inc errlist.inc
color.o: kuplot.inc errlist.inc
config.o: kuplot.inc envir.inc errlist.inc learn.inc
cx11.o: cx11.h
dkdo.o: doact.inc doloop.inc doexec.inc errlist.inc learn.inc macro.inc \
	prompt.inc charact.inc random.inc wink.inc
draw.o: kuplot.inc param.inc errlist.inc
err_appl.o: errlist.inc
errlist.o: errlist.inc
fit.o: kuplot.inc doact.inc macro.inc errlist.inc learn.inc prompt.inc \
	wink.inc
frame.o: kuplot.inc errlist.inc
graph.o: kuplot.inc errlist.inc wink.inc
help.o: prompt.inc envir.inc doact.inc macro.inc errlist.inc
hpgl.o: kuplot.inc errlist.inc
kdo.o: kuplot.inc learn.inc errlist.inc
kuplot.o: errlist.inc doact.inc learn.inc prompt.inc macro.inc wink.inc
learn.o: errlist.inc prompt.inc learn.inc
load.o: errlist.inc kuplot.inc
macro.o: errlist.inc macro.inc prompt.inc charact.inc
math.o: kuplot.inc errlist.inc param.inc wink.inc
para.o: kuplot.inc errlist.inc
plot.o: kuplot.inc errlist.inc wink.inc
ps.o: kuplot.inc errlist.inc wink.inc
random.o: random.inc
save.o: kuplot.inc prompt.inc macro.inc doact.inc errlist.inc learn.inc
show.o: kuplot.inc errlist.inc param.inc
sup.o: random.inc errlist.inc param.inc prompt.inc
sys_f2c.o: wink.inc errlist.inc envir.inc
sys_hpu.o: errlist.inc envir.inc
sys_sun.o: errlist.inc envir.inc
tek.o: kuplot.inc
upd_par.o: kuplot.inc errlist.inc param.inc
x11.o: kuplot.inc wink.inc errlist.inc
