c*****7*****************************************************************
c
c	These generic plot subroutines are called by KUPL and the 
c	corresponding device dependent plot routine is called here.
c
c	Version : 1.0
c	Date    : 11 Oct 96
c
c	Author  : Th. Proffen (proffen@rsc.anu.edu.au)
c-
	subroutine init_kupl (filname)
c
	implicit	none
c
	include		'kuplot.inc'
	include		'errlist.inc'
c
	character*(*)	filname
c
	if (id_dev.eq.hpgl) then
	  call init_hpgl (filname)
	elseif (id_dev.eq.ps) then
	  call init_ps (filname)
	elseif (id_dev.eq.tek) then
	  call init_tek
        elseif (id_dev.eq.x11) then
          call init_x11
	else
	  ier_num = -11
	  ier_typ = ER_APPL
	endif
	end
c
c*****7*****************************************************************
	subroutine open_kupl
c
	implicit	none
c
	include		'kuplot.inc'
c
	if (id_dev.eq.hpgl) then
	  call open_hpgl
	elseif (id_dev.eq.ps) then
	  call open_ps
	elseif (id_dev.eq.tek) then
	  call open_tek
        elseif (id_dev.eq.x11) then
          call open_x11
	endif
	end
c*****7*****************************************************************
	subroutine flush_kupl
c
	implicit	none
c
	include 	'kuplot.inc'
c
	if (id_dev.eq.x11) then
	  call close_x11
	endif
	end
c*****7*****************************************************************
	subroutine close_kupl
c
	implicit	none
c
	include		'kuplot.inc'
c
	if (id_dev.eq.hpgl) then
	  call close_hpgl
	elseif (id_dev.eq.ps) then
	  call close_ps
	elseif (id_dev.eq.tek) then
	  call close_tek
        elseif (id_dev.eq.x11) then
          call close_x11
	endif
	end
c*****7*****************************************************************
	subroutine col_kupl (icol)
c
	implicit	none
c
	include		'kuplot.inc'
c
	integer		icol
c
	if (id_dev.eq.hpgl) then
	  call col_hpgl (icol)
	elseif (id_dev.eq.ps) then
	  call col_ps (icol)
	elseif (id_dev.eq.tek) then
	  call col_tek (icol)
        elseif (id_dev.eq.x11) then
          call col_x11 (icol)
	endif
	end
c*****7*****************************************************************
	subroutine colrgb_kupl (r,g,b)
c
	implicit	none
c
	include		'kuplot.inc'
c
	real		r,g,b
c
	if (id_dev.eq.hpgl) then
	  write (*,1000) 
	elseif (id_dev.eq.ps) then
	  call colrgb_ps (r,g,b)
	elseif (id_dev.eq.tek) then
	  write (*,1000) 
        elseif (id_dev.eq.x11) then
	  call colrgb_x11 (r,g,b)
	endif
c
1000	format(' ****PLOT**** Frame background color not supported for',
     &	       ' current device ****')
	end
c*****7*****************************************************************
	subroutine ltyp_kupl (ityp,lwidth)
c
	implicit	none
c
	include		'kuplot.inc'
c
	integer		ityp
	real 		lwidth
c
	if (id_dev.eq.hpgl) then
	  call ltyp_hpgl (ityp,lwidth)
	elseif (id_dev.eq.ps) then
	  call ltyp_ps (ityp,lwidth)
	elseif (id_dev.eq.tek) then
	  call ltyp_tek (ityp)
        elseif (id_dev.eq.x11) then
          call ltyp_x11 (ityp,lwidth)
	endif
	end
c*****7*****************************************************************
	subroutine fsize_kupl (fsize,fid)
c
	implicit	none
c
	include		'kuplot.inc'
c
	integer		fid
	real		fsize
c
	if (id_dev.eq.hpgl) then
	  call fsize_hpgl (fsize,fid)
	elseif (id_dev.eq.ps) then
	  call fsize_ps (fsize,fid)
	elseif (id_dev.eq.tek) then
	  call fsize_tek (fsize)
        elseif (id_dev.eq.x11) then
          call fsize_x11 (fsize,fid)
	endif
	end
c*****7*****************************************************************
	subroutine text_kupl (xp,yp,text,jmode)
c
	implicit	none
c
	include		'kuplot.inc'
c
	character*(*)	text
	real		xp,yp
	integer		jmode
c
	call koor_trans (1,xp,yp)
	if (id_dev.eq.hpgl) then
	  call text_hpgl (xp,yp,text,jmode)
	elseif (id_dev.eq.ps) then
	  call text_ps (xp,yp,text,jmode)
	elseif (id_dev.eq.tek) then
	  call text_tek (xp,yp,text,jmode)
        elseif (id_dev.eq.x11) then
          call text_x11 (xp,yp,text,jmode)
	endif
	end
c*****7*****************************************************************
	subroutine pline_kupl (npkt,xa,ya)
c
	implicit	none
c
	include		'kuplot.inc'
c
	integer		npkt
	real		xa(npkt),ya(npkt)
c
	call koor_trans (npkt,xa,ya)
c
	if (id_dev.eq.hpgl) then
	  call pline_hpgl (npkt,xa,ya)
	elseif (id_dev.eq.ps) then
	  call pline_ps (npkt,xa,ya)
	elseif (id_dev.eq.tek) then
	  call pline_tek (npkt,xa,ya)
        elseif (id_dev.eq.x11) then
          call pline_x11 (npkt,xa,ya)
	endif
	end
c*****7*****************************************************************
	subroutine rect_kupl (px,py,delx,dely,fill)
c
	implicit	none
c
	include		'kuplot.inc'
c
	real		px,py,delx,dely
	real		xlu,ylu,xro,yro
	logical		fill
c
	call koor_trans (1,px,py)
	xlu=px-delx
	ylu=py-dely
	xro=px+delx
	yro=py+dely
c
	if (id_dev.eq.hpgl) then
	  call rect_hpgl (xlu,ylu,xro,yro,fill)
	elseif (id_dev.eq.ps) then
	  call rect_ps   (xlu,ylu,xro,yro,fill)
	elseif (id_dev.eq.tek) then
	  call rect_tek  (xlu,ylu,xro,yro,fill)
        elseif (id_dev.eq.x11) then
          call rect_x11  (4,xlu,ylu,xlu,yro,xro,yro,xro,ylu,fill)
	endif
	end
c*****7*****************************************************************
	subroutine area_kupl (npkt,xp,yp,fill)
c
	implicit	none
c
	include		'kuplot.inc'
c
	integer		npkt
	real		xp(npkt),yp(npkt)
	logical		fill
c
	call koor_trans (npkt,xp,yp)
c
	if (id_dev.eq.hpgl) then
	  call area_hpgl (npkt,xp,yp,fill)
	elseif (id_dev.eq.ps) then
	  call area_ps   (npkt,xp,yp,fill)
	elseif (id_dev.eq.tek) then
	  call area_tek  (npkt,xp,yp,fill)
        elseif (id_dev.eq.x11) then
          call rect_x11  (npkt,xp,yp,fill)
	endif
	end
c*****7*****************************************************************
	subroutine circ_kupl (px,py,delx,dely,fill)
c
	implicit	none
c
	include		'kuplot.inc'
c
	real		px,py,delx,dely
	logical		fill
c
	call koor_trans (1,px,py)
c
	if (id_dev.eq.hpgl) then
	  call circ_hpgl (px,py,delx,dely,fill)
	elseif (id_dev.eq.ps) then
	  call circ_ps   (px,py,delx,dely,fill)
	elseif (id_dev.eq.tek) then
	  call circ_tek  (px,py,delx,dely,fill)
        elseif (id_dev.eq.x11) then
          call circ_x11  (px,py,delx,dely,fill)
	endif
	end
c*****7*****************************************************************
	subroutine bitmap_kupl (ik)
c
	implicit	none
c
	include		'kuplot.inc'
c
	integer		ik
c
	if (id_dev.eq.hpgl) then
	  write (*,1000)
	elseif (id_dev.eq.ps) then
	  call bitmap_ps(ik)
	elseif (id_dev.eq.tek) then
	  write (*,1000)
	elseif (id_dev.eq.x11) then
	  call bitmap_x11(ik)
	endif
c
1000	format(' ****PLOT**** Bitmap output not supported for',
     &	       ' current device ****')
	end
c*****7*****************************************************************
	subroutine in_kupl (wx,wy,key)
c
	implicit	none
c
	include		'kuplot.inc'
	include		'wink.inc'
c
	character*1	key
	real		wx,wy
c
	if (id_dev.eq.hpgl) then
	  write (*,1000)
	elseif (id_dev.eq.ps) then
	  write (*,1000)
	elseif (id_dev.eq.tek) then
	  call in_tek (wx,wy,key)
	  if (shear(iframe).ne.90.0) then
	    wx=wx-(wy-ey(iframe,1))*yskal(iframe)/tan(rad*shear(iframe))
	  endif
      	elseif (id_dev.eq.x11) then
          call in_x11 (wx,wy,key)
          if (shear(iframe).ne.90.0) then
            wx=wx-(wy-ey(iframe,1))*yskal(iframe)/tan(rad*shear(iframe))
          endif
	endif
c
1000	format(' ****PLOT**** Graphical input not supported for',
     &	       ' current device ****')
	end
