c*****7*****************************************************************
	subroutine do_show (line,lp)
c+
c	This routine shows all settings of KUPLOT parameters. Some
c	parameter settings can also be displayed by entering the
c	corresponding command without parameters.
c
c	Version : 1.0
c	Date    : 13 Oct 96
c
c	Author  : Th. Proffen  (proffen@rsc.anu.edu.au)
c-                                         
	implicit	none
c
	integer		maxw
	parameter	(maxw=2)
c
	include		'kuplot.inc'
	include		'errlist.inc'
c
	character*(*)	line
	character*200	cpara(maxw)
	real		werte(maxw)
	integer		lpara(maxw),lp
	integer		ianz,iku
	logical		str_comp
c
	call get_params(line,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.eq.0) then
c
c------ - Output all information
c
	  call show_data(-1)
	  call show_plot
	  call show_style
	  call show_hlin
	  call show_annotation
	  call show_frames
	  call show_font
c
	elseif (ianz.eq.1 .or. ianz.eq.2) then
c
c------ - Contour line information
c
	  if(str_comp(cpara(1),'hlin',2,lpara(1),4)) then
	    call show_hlin
c
c------ - Data information
c
	  elseif(str_comp(cpara(1),'data',2,lpara(1),4)) then
	    if (ianz.eq.1) then
	      call show_data(-1)
	    else
	      call del_params(1,ianz,cpara,lpara,maxw)
	      call ber_params(ianz,cpara,lpara,werte,maxw)
	      iku=nint(werte(1))
	      call show_data(iku)
	    endif
c
c------ - Plot information
c
	  elseif(str_comp(cpara(1),'plot',2,lpara(1),4)) then
	    call show_plot
c
c------ - Style information (colors, ...)
c
	  elseif(str_comp(cpara(1),'style',2,lpara(1),5)) then
	    call show_style
c
c------ - Show annotations
c
	  elseif(str_comp(cpara(1),'annot',2,lpara(1),5)) then
	    call show_annotation
c
c------ - Current color map
c
	  elseif(str_comp(cpara(1),'color',2,lpara(1),5)) then
	    call show_color
c
c------ - Current frame setting
c
	  elseif(str_comp(cpara(1),'frame',2,lpara(1),5)) then
	    call show_frames
c
c------ - Current font settings
c
	  elseif(str_comp(cpara(1),'font',2,lpara(1),4)) then
	    call show_font
c
c------ - Error status setting
c
	  elseif(str_comp(cpara(1),'error',2,lpara(1),5)) then
	   if(ier_sta.eq.ER_S_CONT) then
	      write(*,2100)
	    else
	      write(*,2105)
	    endif
c
c------ - Result array
c
	  elseif(str_comp(cpara(1),'result',2,lpara(1),6)) then
	    call show_res
c
	  else
	    ier_num = -6
	    ier_typ = ER_COMM
	  endif  
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
2100    format(' Program continues after display of error message')
2105    format(' Program terminates after display of error message')
	end
c*****7*****************************************************************
	subroutine show_hlin
c+
c	Show contour line information
c-
	implicit	none
c
	include		'errlist.inc'
	include		'kuplot.inc'
c
	integer		j
c
	write (*,1000) iframe
	write (*,1010) 
	do j=1,iho(iframe)
	  write (*,1020) j,z_min(iframe,j),z_inc(iframe,j),nz(iframe,j)
	enddo
	write(*,*)
c
1000	format (' Contour line settings for frame no.: ',i3,/)
1010	format (3x,'Cont.set    minimum       ',
     &	        'increment   # of contours',/,3x,56('-'))
1020	format (4x,i3,7x,g11.5,3x,g11.5,6x,i4)
	end
c*****7*****************************************************************
	subroutine show_data(iku)
c+
c	Print information about loaded data
c-                                
	implicit	none
c
	include		'errlist.inc'
	include		'kuplot.inc'
c
	integer		iku,ia,ie,ik
c
c------ check if we want ALL data sets
c
	if(iku.eq.-1) then
	  ia=1
	  ie=iz-1
	else
	  if(iku.gt.iz-1 .or. iku.lt.1) then
	    ier_num = -4
	    ier_typ = ER_APPL
	    return
	  endif
	  ia=iku
	  ie=iku
	endif
c
c------ Print information
c
	do ik=ia,ie
	  if(lni(ik)) then
	    call get_extrema_xy (x,ik,nx(ik),xmin,xmax)
	    call get_extrema_xy (y,ik,ny(ik),ymin,ymax)
	    call get_extrema_z  (z,ik,nx(ik),ny(ik),zmin,zmax)
	    write(*,1000) ik,fname(ik),nx(ik),ny(ik),xmin(ik),xmax(ik),
     &	                  ymin(ik),ymax(ik),zmin(ik),zmax(ik)
	  else
	    call get_extrema_xy (x,ik,len(ik),xmin,xmax)
	    call get_extrema_xy (y,ik,len(ik),ymin,ymax)
	    write(*,1010) ik,fname(ik),len(ik),xmin(ik),xmax(ik),
     & 	                  ymin(ik),ymax(ik)
	  endif
	enddo
c
1000	format (' Data set no.: ',i3,//,
     &            3x,'Filename : ',a35,'(',i3,' * ',i3,' pts )'/
     &            3x,'Range  x : ',g12.4,' to ',g12.4/  
     &            3x,'Range  y : ',g12.4,' to ',g12.4/  
     &            3x,'Range  z : ',g12.4,' to ',g12.4,/)
1010	format (' Data set no.: ',i3,//, 
     &            3x,'Filename : ',a35,'(',i6,' points )'/
     &            3x,'Range  x : ',g12.4,' to ',g12.4/  
     &            3x,'Range  y : ',g12.4,' to ',g12.4,/)
c
	end  
c*****7****************************************************************
	subroutine show_plot
c+
c	Show information about plot settings
c-
	implicit	none
c
	include 	'kuplot.inc'
	include 	'errlist.inc'
c
	write(*,1000) iframe,plmass(iframe),yskal(iframe),shear(iframe),
     &	              ilinecol(0),linewid(0)
c
	if(ibox(iframe).eq.0) then
	  write(*,1005) 'no frame'
	elseif(ibox(iframe).eq.1) then
	  write(*,1005) 'only box around plot'
	elseif(ibox(iframe).eq.2) then
	  write(*,1005) 'box and tick marks / labels'
	elseif(ibox(iframe).eq.3) then
	  write(*,1005) 'box, tick marks and x,y-axis'
	elseif(ibox(iframe).eq.4) then
	  write(*,1005) 'box, tick marks (both sides) and x,y-axis'
	endif
c
        if(ifname(iframe)) then
	  write(*,1010) 'on '
	else
	  write(*,1010) 'off'
	endif
        if(tot_frame) then
	  write(*,1013) 'on '
	else
	  write(*,1013) 'off' 
	endif
        if(igrid(iframe)) then
	  write(*,1015) 'on '
	else
	  write(*,1015) 'off'
	endif
c
1000	format(' Parameter for frame no.: ',i2,//
     &	        3x,'Units of x-axis [cm]     : ',g11.5/
     &	        3x,'Ratio y/x-axis           : ',g11.5/
     &	        3x,'Angle between x&y axis   : ',f7.2/
     &	        3x,'Color of grid lines      : ',i7/
     &	        3x,'Line width of frame [cm] : ',f7.2)
1005	format( 3x,'Type of frame            : ',a)
1010	format( 3x,'Plot filenames           : ',a)
1013	format( 3x,'Plot frame               : ',a)
1015	format( 3x,'Plot grid                : ',a/)
	end
c*****7****************************************************************
	subroutine show_frames
c+
c	Show current frame settings
c-
	implicit	none
c
	include 	'kuplot.inc'
c
	integer		i,j
c
	write (*,1000) iframe
	write (*,2000)
	do i=1,iaf
	  write (*,2500) i,(frame(i,j),j=1,4),(frback(i,j),j=1,3)
	enddo
c
	write (*,3000)
	do i=1,iaf
	  if(infra(i,1).ne.-1) then
	    write (*,3500) i,(infra(i,j),j=1,maxkurv)
	  else
	    write (*,3600) i,ftext(i)
	  endif
	enddo
	write (*,*)
c
1000	format (' Frame settings (active frame : ',i3,') :',/)
2000	format (3x,'Frame  xmin xmax ymin ymax  Backgr.  r   g   b',/
     &	        3x,'------------------------------------------------')
2500	format (3x,i3,4x,4(f4.2,1x),9x,3(f3.1,1x))
3000	format (/,3x,'Frame  data sets/text file',/,3x,67('-'))
3500	format (3x,i3,4x,20i3)
3600	format (3x,i3,4x,a40)
	end
c*****7****************************************************************
	subroutine show_font
c+
c	Show current font settings
c-
	implicit	none
c
	include 	'kuplot.inc'
c
	character*20    fon_name(16)
c
	data    fon_name  /'Times','Courier',
     &	                   'Symbol','Helvetica',
     &	                   'Times bold','Courier bold',
     &	                   'Symbol bold','Helvetica bold',
     &	                   'Times italic','Courier italic',
     &	                   'Symbol italic','Helvetica italic',
     &	                   'Times b&i','Courier b&i',
     &	                   'Symbol b&i','Helvetica b&i'/
c
	write (*,1000) iframe
	write (*,1100) 1,'Main title line    ',fonsize(iframe,1)/10.,
     &	               fon_name(fon_id(iframe,1)),fon_id(iframe,1),
     &	               foncol(iframe,1)
	write (*,1100) 2,'Subtitle line      ',fonsize(iframe,2)/10.,
     &	               fon_name(fon_id(iframe,2)),fon_id(iframe,2),
     &	               foncol(iframe,2)
	write (*,1100) 3,'Axis labels        ',fonsize(iframe,3)/10.,
     &	               fon_name(fon_id(iframe,3)),fon_id(iframe,3),
     &	               foncol(iframe,3)
	write (*,1100) 4,'Numbers at axis    ',fonsize(iframe,4)/10.,
     &	               fon_name(fon_id(iframe,4)),fon_id(iframe,4),
     &	               foncol(iframe,4)
	write (*,1100) 5,'Text in text frame ',fonsize(iframe,5)/10.,
     &	               fon_name(fon_id(iframe,5)),fon_id(iframe,5),
     &	               foncol(iframe,5)
	write (*,1100) 6,'Filename & caption ',fonsize(iframe,6)/10.,
     &	               fon_name(fon_id(iframe,6)),fon_id(iframe,6),
     &	               foncol(iframe,6)
	write (*,*)
c
1000	format (' Font settings for frame no.: ',i3,//,
     &	        3x,'Font  where ?              size   fontname',
     &	        12x,'font-id   color',/,3x,69('-'))
1100	format (3x,i3,3x,a,2x,f4.0,2x,a,4x,i2,7x,i1)
	end
c*****7****************************************************************
	subroutine show_res
c+
c	Show result array
c-
	implicit	none
c
	include 	'param.inc'
c
	integer		i,j,k,k1,k2
c
	if (res_para(0).eq.0) then
	  write(*,1000)
	else
          write(*,2000) nint(res_para(0))
          j = nint(res_para(0))/5
          if(mod(nint(res_para(0)),5).eq.0) then
            j = j-1
          endif
          do i=0,j
            k1 =     5*i+1
            k2 = min(5*i+5,nint(res_para(0)))
            write(*,2010) (res_para(k),k=k1,k2)
          enddo
	  write (*,*) ' '
	endif
c
1000	format (' Result array is empty ')
2000	format (' Result array containing ',i4,' values :',/)
2010	format (3x,5(g13.7,2x))
	end
c*****7****************************************************************
	subroutine show_annotation
c+
c	Show annotations
c-
	implicit	none
c
	include 	'kuplot.inc'
c
	character*1	just
	integer		i,il,len_str
	logical		noann
c
	noann = .true.
c
	write (*,1000) iframe
	do i=1,maxan
	  if (antext(iframe,i)(1:3).ne.'OFF') then
	    if (anjust(iframe,i).eq.if_left)   just='L'
	    if (anjust(iframe,i).eq.if_right)  just='R'
	    if (anjust(iframe,i).eq.if_centre) just='C'
	    il=len_str(antext(iframe,i))
	    write (*,2000) i,anx(iframe,i),any(iframe,i),
     &	                   just,antext(iframe,i)(1:il)
	    noann = .false.
	  endif
	enddo
c
	if (noann) write (*,3000)
c
1000	format (' Annotation for frame no.: ',i3,//
     &	        3x,'  #',6x,'x',13x,'y',8x,'just  text',/,3x,69('-'))
2000	format (3x,i3,2(2x,g12.6),2x,a1,4x,a)
3000	format ('   *** no annotations set ***')
	end
c*****7****************************************************************
	subroutine show_color
c+
c	Show current color map
c-
	implicit	none
c
	include 	'kuplot.inc'
c
	integer		i,j
c
	write (*,1000) 
	write (*,2000) (i,(col_map(i,j),j=1,3),i=1,maxcol)
c
1000	format (/,' Current color map values (RGB) : ',/)
2000	format (4(3x,i3,':',3(1x,f3.1)))
	end
c*****7****************************************************************
	subroutine show_style
c+
c	Show information about plot styles (color, ..)
c-
	implicit	none
c
	include 	'kuplot.inc'
	include 	'errlist.inc'
c
	character*40	cdummy
	integer		i,j,ik
	logical		d2,d3
c
	integer		len_str
c
	d3 = .false.
	d2 = .false.
	do i=1,iz-1
	  d3 = d3 .or. lni(i)
	  d2 = d2 .or. .not.lni(i)
	enddo
c
c------ 2D data set parameters
c
	if (d2) then
	  write (*,1000)
	  do ik=1,iz-1
	    if (.not. lni(ik)) then
	      cdummy='off'
	      if (ilegend(ik).eq.1) cdummy=clegend(ik)
	      write(*,1100) ik,ilinecol(ik),ilinetyp(ik),
     &	                    ilineart(ik),imarkcol(ik),
     &	                    imarktyp(ik),imarkmax(ik),ierr(ik),
     &	                    linewid(ik),sizemark(ik),
     &	                    cdummy(1:len_str(cdummy))
	    endif
	  enddo 
	  write (*,*)
	endif
c
c------ 3D data set parameters
c
	if (d3) then
	  write (*,2000)
	  do ik=1,iz-1
	    if (lni(ik)) then
	      cdummy='off'
	      if (ilegend(ik).eq.1) cdummy=clegend(ik)
	      write(*,2200) ik,hlineart(ik),(hlinecol(ik,j),j=1,maxhl),
     &	                    (hlinetyp(ik,j),j=1,maxhl),
     &	                    linewid(ik),cdummy(1:len_str(cdummy))
	    endif
	  enddo
	  write (*,*)
	endif
c
1000	format(' Plot style for 2D data sets : ',//,
     &	       3x,'d.set  lcol  ltyp  lart  mcol  mtyp  ptyp  etyp',
     &	       '   lwid   msiz   caption',/,3x,76('-'))
1100	format( 3x,i3,1x,7(4x,i2),3x,f5.2,2x,f5.2,3x,a) 
2000	format(' Plot style for 3D data sets : ',//,
     &         3x,'d.set',2x,'hart',5x,'hcol',9x,'htyp',8x,'lwid',
     &	       4x,'caption',/,3x,76('-'))
2200	format(3x,i3,5x,i2,4x,4(i1,','),i1,4x,4(i1,','),i1,3x,f5.2,3x,a13)
c
	end
